<?php
/**
* This coordinates the parser and renderer. 
* 
* @author		Ulf Wendel <ulf@redsys.de>
* @version 	0.1alpha
*/
class Phpdoc extends PhpdocSetupHandler {

	/**
	* Class tree
	* @var		array $classTree
	* @access	private
	*/								
	var $classTree = array();				
	
	/**
	* Array mapping Classnames to Filenames
	* 
	* The array is indexed by the Classnames
	*
	* @var		array		$classnamesToFilenames
	* @access	private	
	*/
	var $classnamesToFilenames = array();
	
	/**
	* Parsing result. Array of all classes
	* @var		array		$classes
	* @access	private	
	*/
	var $classes = array();

	/**
	* Calls the command line handler if necessary
	* @see	handleArgv()
	*/
	function Phpdoc() {
		global $argc;
		if ($argc>1) 
			$this->handleArgv();
	} // end constructor

	
	/**
	* Starts the parser. 
	*
	* Extracts documentation, using some parser objects. 
	* The results are saved in objectvariables, 
	* a classtree is build, private and public data
	* gets isolated, inherited and overriden data is
	* computed.
	*
	* @return	bool		$ok
	* @throws	runtime
	* @access	public
	* @see		buildClassTree(), findInheritedAndOverriden(), updateAccessReturn(),
	*					findPublicPrivate(), $classes, $classTree, $classnamesToFilenames
	*/
	function parse() {
		
		
		// "setup" fine?
		$errors = $this->checkStatus();
		if (0!=count($errors)) {
			reset($errors);
			while (list($k, $error)=each($errors))
				$this->err = new exception_runtime($error["msg"], $error["errno"], __FILE__, __LINE__);
			return false;
		}

		$this->outl("Parser starts...");
		$this->outl("... core work");
		
		// create some objects
		$fileHandler 		= new PhpdocFileHandler;
		$parser 				= new PhpdocParser;

		if (""!=$this->directory) 
			$this->files = $fileHandler->getDirectory($this->directory);
			
		$contents = $fileHandler->Get($this->files);
		
		$parser->setPhpSourcecodeFiles($contents);
		$this->classes = $parser->parse();
		$this->classnamesToFilenames = $parser->getClassnamesToFilenames();
		
		
		$this->outl("... building class tree");
		$this->buildClassTree();
		
		$this->outl("... updating using the new informations from the class tree");
		$this->findInheritedandOverriden();
		$this->updateAccessReturn();
		
		$this->outl("... sorting data by their 'type' (public, private, undocumented)");
		$this->findPublicPrivate();
		
		$this->outl("Parser finished.");
		return true;
	} // end func parse
	
	/**
	* Builds the classtree, computes the "path" and the direct known subclasses
	* @access	private
	*/	
	function buildClassTree() {
	
		// work on all classes...
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {

			$path = array();
			$baseclass = $classname;
			$basefile	 = $filename;
			
			// look for parents
			while ($extends = $this->classes[$basefile][$baseclass]["extends"]) {
				$path[] 		= $extends;
				$baseclass 	= $extends;
				$basefile		= $this->classnamesToFilenames[$baseclass];
			} 
			$this->classes[$filename][$classname]["path"] = $path;
			
			$base = (0==count($path)) ? true : false;
			
			$this->classTree[$classname]["base"] = base;
			$this->classTree[$classname]["path"] = $path;
		} // end while this->classnamesToFilenames
		
		// walk through the tree and find the subclasses
		reset($this->classTree); 
		while (list($classname, $tree)=each($this->classTree)) {
			// skip base classes
			if (!is_array($tree["path"])) 
				continue;
			
			// build a list of subclasses	
			reset($tree["path"]);
			while (list($k, $baseclass)=each($tree["path"])) {
				$this->classTree[$baseclass]["subclasses"][$classname] = true;
				$classname = $baseclass;
			}
		} 
		
		ksort($this->classTree);
	} // end func buildClassTree
	
	/**
	* Looks for inherited and overriden functions and variables
	* and copies them into the class list array
	* @access	private
	* @see		findInheritedVariables()
	*/
	function findInheritedandOverriden() {
		
		$functiontree = array();
		$variabletree = array();
		
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
		
			$tree = $this->classTree[$classname];
			// skip base classes
			if (true == $tree["base"])
				continue;
			
			reset($tree["path"]);
			while (list($k, $parentclass)=each($tree["path"])) {
			
				$parentfilename = $this->classnamesToFilenames[$parentclass];
				$parentfunctions = $this->classes[$parentfilename][$parentclass]["functions"];
				$parentvariables = $this->classes[$parentfilename][$parentclass]["variables"];
				
				// look for inherited and overriden functions
				if (is_array($parentfunctions)) {
				
					// can't override or inherit a constructor...
					unset($parentfunctions[$parentclass]);
					
					reset($parentfunctions);
					while (list($functionname, $data)=each($parentfunctions)) {
					
						if (!isset($this->classes[$filename][$classname]["functions"][$functionname])) {
						
							$this->classes[$filename][$classname]["inherited"]["functions"][$parentclass][$functionname] = true;
							
						} else {
						
							$this->classes[$filename][$classname]["overriden"]["functions"][$parentclass][$functionname] = true;
							$this->classes[$filename][$classname]["functions"][$functionname]["baseclass"] = $parentclass;
							if (!isset($this->classes[$parentfilename][$parentclass]["functions"][$functionname]["undocumented"])) 
								$functiontree[$filename][$classname][$functionname][] = $parentclass;
								
						}
						
					} // end while $parentfunctions
					
				} // end if is_array($parentfunctions)

				// look for inherited and overriden variables
				if (is_array($parentvariables)) {
				
					reset($parentvariables);
					while (list($varname, $data)=each($parentvariables)) {
					
						if (!isset($this->classes[$filename][$classname]["variables"][$varname])) {
						
							$this->classes[$filename][$classname]["inherited"]["variables"][$parentclass][$varname] = true;
							
						} else {
						
							$this->classes[$filename][$classname]["overriden"]["variables"][$parentclass][$varname] = true;
							$this->classes[$filename][$classname]["variables"][$varname]["baseclass"] = $parentclass;
							if (!isset($this->classes[$parentfilename][$parentclass]["variables"][$varname]["undocumented"]))
								$variabletree[$filename][$classname][$varname][] = $parentclass;
								
						}
						
					} // end while $parentvariables
					 
				} // end if is_array($parentvariables)
							
			} // end while %tree["path"]
				
		} // end while $this->classnamesToFilenames
		
		
		// now that we know the overriden functions and variables
		// we can copy some inherited data to the classes and 
		// build a list of overriden functions
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
			
			$tree = $functiontree[$filename][$classname];
			if (is_array($tree)) {
			
				reset($tree);
				while (list($functionname, $path)=each($tree)) {
				
					reset($path);
					while (list($k, $parentclass)=each($path)) {
						
						// get the data of the parentclasses functions
						$data = $this->classes[$filename][$parentclass]["functions"][$functionname];
						if (!is_array($data))
							continue;
							
						reset($data);
						while (list($k, $v)=each($data)) {
						
							if ( !isset($this->classes[$filename][$classname]["functions"][$functionname]["overriden"][$k]) && "overriden"!=$k) {
							
								// copy some datafields to the current class
								if (!isset($this->classes[$filename][$classname]["functions"][$functionname][$k]))
									$this->classes[$filename][$classname]["functions"][$functionname][$k] = $v;						
									
								// save that this function overrides another one and remember the source of the inherited datafields
								$this->classes[$filename][$classname]["functions"][$functionname]["overriden"][$k]["parentclass"] = $parentclass;
								
							} // end if isset
							
						} // end while $data
						
					} // end while $tree
					
				} // end if is_array($tree)
				
			} // end while each($this->classnamesToFilenames)

			// play it again with the variables, Sam...			
			$tree = $variabletree[$filename][$classname];
			if (is_array($tree)) {
			
				reset($tree);
				while (list($varname, $path)=each($tree)) {
				
					reset($path);
					while (list($k, $parentclass)=each($path)) {
						
						// get the data of the parentclasses variables
						$data = $this->classes[$filename][$parentclass]["variables"][$varname];
						
						reset($data);
						while (list($k, $v)=each($data)) {
						
							if (!isset($this->classes[$filename][$classname]["variables"][$varname]["overriden"][$k]) && "overriden"!=$k) {
							
								// copy some datafields to the current class
								if (!isset($this->classes[$filename][$classname]["variables"][$varname][$k]))							
									$this->classes[$filename][$classname]["variables"][$varname][$k] = $v;
									
								// save that this variables overrides another one and remember the source of the inherited datafields
								$this->classes[$filename][$classname]["variables"][$varname]["overriden"][$k]["parentclass"] = $parentclass;
								
							} // end isset(...)
							
						} // end while $data
						
					} // end while $path
					
				} // end while $tree
				
			} // end if is_array($tree)
						
		}
		
	} // end func findInheritedandOverriden
	
	/**
	* Updates access and return parameters of functions and variables.
	* 
	* Phpdoc copies return and access specifications from parentclasses to
	* subclasses or looks for default values.
	* 
	* @access	private
	* @see		updateFunctions(), updateVariables
	*/
	function updateAccessReturn() {
	
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
			$this->updateFunctions($filename, $classname);
			$this->updateVariables($filename, $classname);		
		}
		
	} // end func updateAccessReturn
	
	/**
	* Update access parameters of variables.
	* @param	string	$filename
	* @param	string	$classname
	* @access	private
	* @see	updateFunctions(), updateAccessReturn()
	*/
	function updateVariables($filename, $classname) {
	
		$variables = $this->classes[$filename][$classname]["variables"];
		if (!is_array($variables)) 
			return;
		
		reset($variables);
		while (list($varname, $data)=each($variables)) {
			if (isset($data["access"])) 
				return;
			
			if (isset($data["baseclass"])) {
			
				$basefilename = $this->classnamesToFilenames[$data["baseclass"]];
				$access = $this->classes[$basefilename][$data["baseclass"]]["variables"][$varname]["access"];
				if (isset($this->classes[$basefilename][$data["baseclass"]]["variables"][$varname]["undocumented"])) {
					$access = "undocumented";
				} else if (""==$access) {
					$access = "private";
				}
					
			} else if (!isset($data["undocumented"])) {
				$access = "private";
		 	}	else  {
				$access = "undocumented";
			}
			
			$this->classes[$filename][$classname]["variables"][$varname]["access"] = $access;
				
		}	
	} // end func updateVariables
	
	/**
	* Updates return and access parameters of functions.
	* @param	string	$filename
	* @param	string	$classname
	* @access	private 
	* @see		updateVariables(), updateAccessReturn()
	*/
	function updateFunctions($filename, $classname) {
	
		$functions = $this->classes[$filename][$classname]["functions"];
		if (!is_array($functions))
			return;
		
		reset($functions);
		while (list($functionname, $data)=each($functions)) {
		
			if (!isset($data["access"])) {
			
				if (isset($data["baseclass"])) {
				
					$basefilename = $this->classnamesToFilenames[$data["baseclass"]];
					$access = $this->classes[$basefilename][$data["baseclass"]]["functions"][$functionname]["access"];
					if (isset($this->classes[$basefilename][$data["baseclass"]]["functions"][$functionname]["undocumented"])) {
						$access = "undocumented";
					} else if (""==$access) {
						$access = "private";
					}
						
				} else if (!isset($data["undocumented"])) {
					$access = "private";
			 	}	else  {
					$access = "undocumented";
				}
				
				$this->classes[$filename][$classname]["functions"][$functionname]["access"] = $access;
			}
				
			if (!isset($data["return"])) {
			
				if (isset($data["baseclass"])) {
				
					$basefilename = $this->classnamesToFilenames[$data["baseclass"]];
					$return = $this->classes[$basefilename][$data["baseclass"]]["functions"][$functionname]["return"];
					if (isset($this->classes[$basefilename][$data["baseclass"]]["functions"][$functionname]["undocumented"])) {
						$return["type"] = "undocumented";
					} else if (""==$return) {
						$return["type"] = "void";
					}
					
				} else if (!isset($data["undocumented"])) {
					$return["type"] = "void";
				} else {
					$return["type"] = "undocumented";
				}
				
				$this->classes[$filename][$classname]["functions"][$functionname]["return"] = $return;
			}
				
		}
		
	} // end func updateFunctions
	
	/**
	* Extracts public and private variables and functions of a class and copies the 
	* list to the classdata
	* @access	private
	*/
	function findPublicPrivate() {
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
			
			$functions = $this->classes[$filename][$classname]["functions"];
			if (is_array($functions)) {
				
				$sorted = array(
													"public" 				=> array(),
													"private"				=> array(),
													"undocumented"	=> array()
											);				

				reset($functions);
				while (list($functionname, $data)=each($functions)) 
					if (""!=$data["access"])
						$sorted[$data["access"]][] = $functionname;
					else
						$sorted["undocumented"][] = $functionname;
				
				ksort($sorted["public"]);
				ksort($sorted["private"]);
				ksort($sorted["undocumented"]);
				$this->classes[$filename][$classname]["sorted_functions"] = $sorted;				
			} 
			
			$variables = $this->classes[$filename][$classname]["variables"];
			if (is_array($variables)) {
			
				$sorted = array(
													"public" 				=> array(),
													"private"				=> array(),
													"undocumented"	=> array()
											);				

				reset($variables);
				while (list($varname, $data)=each($variables))
					if (""!=$data["access"]) 
						$sorted[$data["access"]][] = $varname;
					else
						$sorted["undocumented"][] = $varname;
				
				ksort($sorted["public"]);
				ksort($sorted["private"]);
				ksort($sorted["undocumented"]);
				$this->classes[$filename][$classname]["sorted_variables"] = $sorted;
			}  
			
		}
	} // end func findPublicPrivate
	
	/**
	* Renders the PHPDoc Files. 
	* @param	string 	$targetdirectory
	* @return	bool		$ok
	* @throws	runtime_wrongparam
	* @access	public
	*/
	function render($targetdirectory) {
		if (""==$targetdirectory) {
			$this->err = new exception_runtime_wrongparam("No targetdirectory given.", 8, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		
		if ("/"!=substr($targetdirectory, -1))
			$targetdirectory.= "/";
			
		$fileHandler = new PhpdocFileHandler;
		$index			 = "<ul>";
		
		$this->outl("Renderer starts...");
		flush();
		
		reset($this->classTree);
		while (list($classname, $tree)=each($this->classTree)) {
			
			$this->out("... working on class '$classname' ");		
			$filename = $this->classnamesToFilenames[$classname];
			$classRenderer = new PhpdocClassRenderer($classname, $this->classes[$filename][$classname], $tree);

			$html = "";
			$html.= $classRenderer->getTree();
			$html.= $classRenderer->getSubclasses();
			$html.= $classRenderer->getDescription();
			$html.= $classRenderer->getConstructor();
			$html.= $classRenderer->getFunctionSummary();
			$html.= $classRenderer->getInheritedFunctions();
			$html.= $classRenderer->getVariableSummary();
			$html.= $classRenderer->getInheritedVariables();
			$html.= $classRenderer->getFunctionDetaillist();
			$html.= $classRenderer->getVariableDetaillist();
			
			$ok = $fileHandler->createFile($classname.".html", $html, $targetdirectory);			
			if (true == $ok) {
				$this->outl("ok");
				$index.= sprintf('<li><a href="%s.html">%s.html</a></li>%s',
														$classname,
														$classname,
														$this->CR_HTML
												);
			} else {
				$this->outl("error");
				$index.= "<li>Error: $classname</li>".$this->CR_HTML;
			}

		}
		
		$ok = $fileHandler->createFile("index.html", $index, $targetdirectory);
		if (true == $ok) 
			$this->outl("Finished.<p>".$this->CR_HMTL.$index);
		else 
			$this->outl(sprintf('Could not generate Overview. Directory: <a href="%s">%s</a>',
														$targetdirectory,
														$targetdirectory
													)
									);

		return $ok;
	} // end func	render
	
} // end class Phpdoc
?>