<?php
/**
* File handling functions in phpdoc. 
*
* @version	0.1alpha
* @author		Ulf Wendel <ulf@redsys.de>
*/
class PhpdocFileHandler extends PhpdocObject {
	
	/**
	* Filepath. The path is automatically added in front of all filenames
	* @var		string	$path
	* @see		setFilePath()
	* @access	private
	*/
	var $path = "";
		
	/**
	* Reads a list of files or one file.
	* @param	string	$files	filename
	* @param	array		$files	list of filenames, $k => $filename
	* @return	array		$contents file contents indexed by filename
	* @throws runtime
	* @access	public
	*/		
	function Get($files) {
		if (""==$files) {
			$this->err = new exception_runtime("Get(), no files specified.", 1, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return array("", "");
		}
		
		if (!is_array($files))
			$files = array($files);
		
		$contents = array();	
		$together = "";
		
		reset($files);
		while (list($k, $filename)=each($files)) 
			$contents[$filename] = $this->getFile($filename);
		
		return $contents;
	} // end func Get
	
	/**
	* Returns an array of filenames in a directory
	* @param	string	$directory	name of directory to scan
	* @return	array		$files
	* @throws	runtime, io
	* @access	public
	*/
	function getDirectory($directory) {
		if (""==$directory) {	
			$this->err = new exception_runtime("getDirectory(), no directory specified.", 5, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return array();
		}	
		if (!is_dir($directory)) {
			$this->err = new exception_io("getDirectory(), '$directory' is no directory.", 6, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return array();				
		}
		
		$handle=@opendir($directory); 
		if (!$handle) {
			$this->err = new exception_io("getDirectory(), can't read directory '$directory'.", 7, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return array();
		}
		
		if ("/"!=substr($directory, -1))
			$directory.="/";
		
		while ($file = readdir($handle)) 
		    if (is_file($directory.$file))
	    			$files[] = $directory.$file;
		closedir($handle); 
		
		#$this->setFilePath($directory);
		
		return $files;
	} // end func getDirectory
	
	/**
	* Sets the filepath. The path is automatically added in front of all filenames
	* @param	string	$path
	* @return	bool	$ok
	* @access	public
	*/
	function setFilePath($path) {
		$this->path = $path;
	} // end func setFilePath
	
	/** 
	* Reads a file. 
	* @param	string	$filename
	* @return	string	$content
	* @throws	runtime, io
	* @access	private
	*/
	function getFile($filename) {
		if (""==$filename) {
			$this->err = new exception_runtime("getFile(), no filename specified.", 2, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);				
			return "";
		}
		if (!file_exists($filename)) {
			$this->err = new exception_io("getFile(), unknown file '$filename'.", 3, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return "";
		}
		if (!$fh = @fopen($filename, "r")) {
			$this->err = new exception_io("getFile(), can't open file '$filename' for reading.", 4, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return "";
		}

		$content = fread($fh, filesize($filename));
		fclose($fh);
		return $content;			
	} // end func getFile
	
	/**
	* Creates a new file.
	* 
	* Create or overrides a file in a specified directory. If the
	* directory does not exists, it attempts to create it.
	*
	* @param	string	$filename
	* @param	string	$content
	* @param	string	$directory	optional name of a directory
	* @return bool		$ok
	* @throws	runtime_wrongparam, io
	* @access	public
	*/ 
	function createFile($filename, $content, $directory="") {
		if (""==$filename || ""==$content) {
			$this->err = new exception_runtime_wrongparam("No filename or no content given.", 8, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		
		if (""!=$directory) {
			if (file_exists($directory) && !is_dir($directory)) {
				$this->err = new exception_io("Target directory exists but is no directory. Won't override it.", 9, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
				return false;				
			}		
			if (!file_exists($directory)) {
				if (!@mkdir($directory, 0700)) {
					$this->err = new exception_io("Can't create target directory '$directory'. No access?", 10, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
					return false;
				}
			}
		
			if ("/"!=substr($directory, -1))
				$directory.="/";
					
			$filename = $directory.$filename;
		}
		
		$fh = @fopen($filename, "w");
		if (!$fh) {
			$this->err = new exception_io("Can't create file '$filename'.", 11, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		
		fwrite($fh, $content);
		fclose($fh);
		
		return true;
	} // end func createFile
	
		
} // end class PhpdocFileHandler
?>