<?php
/**
* Renders classes
* @author		Ulf Wendel <ulf@redsys.de>
* @version  0.1
*/
class PhpdocClassRenderer extends PhpdocRendererObject {

	/**
	* Name of the current class.
	* @var		string	$classname
	* @see		PhpdocClassRenderer()
	*/
	var $classname 	= "";
	
	/**
	* Array with the basic data of the current class
	* @var		array		$classdata
	* @see		PhpdocClassRenderer()
	*/
	var $classdata = array();
	
	/**
	* Array with a list of parentclasses.
	* @var		array		$tree
	* @see		
	*/
	var $tree = array();
	
	/**
	* Shared PhpdocFunctionRenderer object.
	* @var		object	PhpdocFunctionRenderer	$functionRenderer
	* @see		PhpdocClassRenderer()
	*/
	var $functionRenderer;
	
	/**
	* Shared PhpdocVariableRenderer object.
	* @var		object	PhpdocVariableRenderer	$variableRenderer
	* @see		PhpdocClassRenderer()
	*/
	var $variableRenderer;

	/**
	* Loads the object with data about the current class.
	*
	* The constructor has two jobs. It loads the object with data about
	* the current class and it creates a shared function and variables renderer.
	* 
	* @param	string	$classname	
	* @param	array		$classdata
	* @param	array		$tree
	* @see	functionRenderer, variableRenderer
	*/	
	function PhpdocClassRenderer($classname, $classdata, $tree) {
	
		$this->classname 					= $classname;
		$this->classdata					= $classdata;
		$this->tree								= $tree;
		
		$this->functionRenderer 	= new PhpdocFunctionRenderer;
		$this->variableRenderer		= new PhpdocVariableRenderer;
		
	} // end constructor
	
	/**
	* Renders the inheritance chain as a tree.
	* @return	string	$html
	* @access	public
	*/
	function getTree() {
		
		$path 		= $this->tree["path"];
		// add the current class "in front of" all parentclasses, if there're some
		$path[-1] = $this->classname;
		
		$html  		= "<h1>Class ".$this->classname."</h1>".$this->CR_HTML."<pre>";
		$level		= 0;
		$indent 	= "";
		
		krsort($path);
		list($k, $classname)=each($path);
		$html.= sprintf('<a href="%s.html">%s</a>%s', $classname, $classname, $this->CR_HTML);
		
		while (list($k, $classname)=each($path)) {
			$indent = $this->getIndent($level+=2);	
			$html.= sprintf('%s|%s', $indent, $this->CR_HTML);
			$html.= sprintf('%s+-- <a href="%s.html">%s</a>%s', $indent, $classname, $classname, $this->CR_HTML);
		}
		$html.="</pre><p>\n";
		
		return $html;
	} // end func getTree
	
	/**
	* Renders a list of known direct cubclasses if there're subclasses.
	* @return	string	$html
	* @throws	runtime_wrongparam
	* @access	public
	*/ 
	function getSubclasses() {
		
		if (!isset($this->tree["subclasses"]))
			return "";
		
		$subclasses = $this->tree["subclasses"];
		$html 			= "<h4>Direct Known Subclasses:</h4><code>\n";
		
		reset($subclasses);
		while (list($classname, $v)=each($subclasses)) 
			$html.= sprintf('<a href="%s.html">%s</a>, %s', 
													$classname,
													$classname,
													$this->CR_HTML
											);
		$html = substr($html, 0, -1*(strlen($this->CR_HTML)+2) )."</code>".$this->CR_HTML;
		
		return $html;
	} // end func getSubclasses
	
	/**
	* Renders the class description.
	* @return	string	$html
	* @access	public
	*/
	function getDescription() {
		
		$html = "<h4>Description</h4>";
		$html.= (isset($this->classdata["undocumented"])) ? "Undocumented." : $this->classdata["desc"];		
		$html.= "<p>".$this->CR_HTML;
		
		$attribs =  array(	
												"File"						=> "filename",
												"Version"					=> "version", 
												"Supported since"	=> "since"
									);
									
		reset($attribs);
		while (list($label, $attrib)=each($attribs)) 
			if (isset($this->classdata[$attrib]))
				$html.= sprintf("%s: %s<br>%s", $label, $this->classdata[$attrib], $this->CR_HTML);
		
		if (isset($this->classdata["author"])) {
			$this->classdata["author"] = preg_replace("/<([^>]*)>/", '<a href="mailto:\1">&lt;\1&gt;</a>', $this->classdata["author"]);
			$html.= sprintf("Author(s): %s<br>%s", $this->classdata["author"], $this->CR_HTML);
		}
		
		return $html;
	} // end getDescription
	
	/**
	* Renders the Constructor (in detail)
	* @return	string	$html
	* @access	public
	*/
	function getConstructor() {
		if (!isset($this->classdata["functions"][$this->classname]))
			return "";
		
		$html = "<h4>Constructor</h4>".$this->CR_HTML;
		$html.= $this->functionRenderer->getFunctionDetail($this->classname, $this->classdata["functions"][$this->classname]);
		return $html;
	} // end func getConstructor
	
	/**
	* Renders an overview of all methods.
	* @return	string	$html
	* @access	public
	* @see		getFunctionList()
	*/
	function getFunctionSummary() {
		return $this->getFunctionList("Method Summary", false);
	} // end func getFunctionSummary
	
	/**
	* Renders all methods in detail view.
	* @return	string	$html
	* @access	public
	* @see		getFunctionList()
	*/
	function getFunctionDetaillist() {
		return $this->getFunctionList("Detailed Method List", true);									
	} // end func getFunctionDetaillist

	/**
	* Renders all functions as an overview or in detail.
	* @param	string	$label
	*	@param	bool		$detail
	* @see		getFunctionSummary(), getFunctionDetailParagraph()
	* @access	private
	*/
	function getFunctionList($label, $detail) {
	
		$types 			= array ("public", "private", "undocumented");
		$functions	= array();
		$num				= 0;
		
		reset($types);
		while (list($k, $type)=each($types)) {
			$functions[$type] = $this->getSortedFunctions($type);
			
			reset($functions[$type]);
			while (list($k, $functionname)=each($functions[$type])) 
				if ($functionname == $this->classname) {
					unset($functions[$type][$k]);
					break;
				}
			$num+=count($functions[$type]);
		}
		
		// I'm sorry, no functions (but possibly a constructor) in this class
		if (0==$num)
			return "";				

		// some labels for certain types of functions
		$labels = array(
										"public"				=> "Public Methods",
										"private"				=> "Private Methods",
										"undocumented"	=> "Undocumented Methods"
									);
			
		$html = "<h4>$label</h4><p>".$this->CR_HTML;
		
		if (false == $detail) {
		
			// render a summary
			$html.= "<table>".$this->CR_HTML;			
			
			reset($types);
			while (list($k, $type)=each($types)) 
				if (0!=count($functions[$type]))
					$html.= $this->getFunctionSummaryParagraph($labels[$type], $functions[$type]);
			$html.="</table>".$this->CR_HTML;
			
		} else {
			
			// render every function in detail...
			reset($types);
			while (list($k, $type)=each($types)) 
				if (0!=count($functions[$type]))
					$html.= $this->getFunctionDetailParagraph($labels[$type], $functions[$type]);
		}
		
		return $html;
	} // end func getFunctionList

	/**
	* Renders a list of inherited methods.
	* @return	string	$html
	* @access	public
	*/
	function getInheritedFunctions() {
	
		$inherited 			= $this->classdata["inherited"]["functions"];
		$num_inherited 	= (is_array($inherited)) ? count($inherited) : 0;
		
		if (0 == $num_inherited) 
			return "";
		
		// don't show inherited functions twice, remember which ones are already shown
		$showed = array();

		$html	= "<h4>Inherited Methods</h4>".$this->CR_HTML."<ul>".$this->CR_HTML;			
		$path = "";
			
		reset($inherited);
		while (list($parentclass, $functions)=each($inherited)) {
			
			$path.= $parentclass."/";
			$flist= "";
			
			$html.= sprintf('<li><h5>From <a href="%s.html">%s</a>:</h5><br>%s<code>',
												substr($path, 0, -1),
												$parentclass,
												$this->CR_HTML
											);
											
			ksort($functions);				
			while (list($functionname, $v)=each($functions)) 
				if (!isset($showed[$functionname])) {
					$flist.= sprintf('<a href="%s.html#%s()">%s()</a>,%s ',
															$parentclass,
															strtolower($functionname),
															$functionname,
															$this->CR_HTML
														);
					$showed[$functionname] = true;														
				}
			
			$html.= substr($flist, 0, -1*(strlen($this->CR_HTML)+2))."</code>";
		}

		$html.="</ul>".$this->CR_HTML;
		return $html;
	} // end func getInheritedFunctions
	
	/**
	* Renders a list of inherited variables.
	* @return	string	$html
	* @access	public
	*/
	function getInheritedVariables() {

		$inherited 	= $this->classdata["inherited"]["variables"];
		$num_inherited = (is_array($inherited)) ? count($inherited) : 0;
		if (0 == $num_inherited) 
			return "";
		
		// don't show inherited variables twice, remenber which ones are already shown
		$showed = array();
		$html	= "<h4>Inherited Variables</h4>".$this->CR_HTML."<ul>".$this->CR_HTML;
		$path = "";
			
		reset($inherited);
		while (list($parentclass, $variables)=each($inherited)) {
				
			$path.= $parentclass."/";
			$flist= "";
			
			$html.= sprintf('<li><h5>From <a href="%s.html">%s</a>:</h5><br>%s<code>',
												substr($path, 0, -1),
												$parentclass,
												$this->CR_HTML
											);
											
			ksort($variables);				
			while (list($varname, $v)=each($variables)) 
				if (!isset($showed[$varname])) {
					$flist.= sprintf('<a href="%s.html#%s">%s</a>,%s ',
															$parentclass,
															$varname,
															$varname,
															$this->CR_HTML
														);
					$showed[$varname] = true;														
				}
			
			$html.= substr($flist, 0, -1*(strlen($this->CR_HTML)+2))."</code>";
		}

		$html.="</ul>".$this->CR_HTML;
		return $html;
	} // end func getInheritedVariables
	
	/**
	* Renders an overview of all variables.
	* @return	string	$html
	* @access	public
	* @see		getVariablelist()
	*/
	function getVariableSummary() {
		return $this->getVariablelist("Variable Summary", false);	
	} // end func getVariableSummary
	
	/**
	* Renders a list of all variables in detail view.
	* @return	string	$html
	* @access	public	
	* @see		getVariablelist()
	*/
	function getVariableDetaillist() {
		return $this->getVariablelist("Detailed Variable List", true);
	} // end func getVariableDetaillist
	
	/**
	* Renders a list of all variables eigther as an overview or in detail.
	* @param	string	$label
	* @param	bool		$detail
	* @access	private
	*/
	function getVariablelist($label, $detail) {
	
		$types 			= array ("public", "private", "undocumented");
		$variables	= array();
		$num				= 0;
		
		// get all kinds of variables
		reset($types);
		while (list($k, $type)=each($types)) {
			$variables[$type] = $this->getSortedVariables($type);
			$num+=count($variables[$type]);
		}

		// I'm sorry, there're no variables
		if (0==$num)
			return "";			
		
		$html = "<h4>$label</h4><p>".$this->CR_HTML;
		$labels = array(
										"public"				=> "Public Variables",
										"private"				=> "Private Variables",
										"undocumented"	=> "Undocumented Variables"
									);


		if (false == $detail) {
			
			// render a summary		
			$html.= "<table>".$this->CR_HTML;			
			reset($types);
			while (list($k, $type)=each($types)) 
				if (0!=count($variables[$type]))
					$html.= $this->getVariableSummaryParagraph($labels[$type], $variables[$type]);
			$html.="</table>".$this->CR_HTML;
			
		} else {
			
			// show them all in detail
			reset($types);
			while (list($k, $type)=each($types)) 
				if (0!=count($variables[$type]))
					$html.= $this->getVariableDetailParagraph($labels[$type], $variables[$type]);
		}
		
		return $html;
	} // end func getVariablelist
	
	/**
	* Returns a paragraph for the variable summary list. 
	* @param	string	$label
	* @param	array		$variables
	* @access	private
	* @see		getFunctionSummary(), getVariableSummary()
	*/	
	function getVariableSummaryParagraph($label, $variables) {
		
		$html = sprintf('<tr><td colspan="3" align="left" valign="top"><b>%s</b></td></tr>%s',
												$label,
												$this->CR_HTML
											);

		reset($variables);
		while (list($k, $variable)=each($variables)) {
			list($return, $overrides, $shortdesc)= $this->variableRenderer->getVariableSummary($variable, $this->classdata["variables"][$variable] );
			$html.= sprintf('<tr>%s<td align="left" valign="top">%s</td>%s',
												$this->CR_HTML,
												$return, 
												$this->CR_HTML
											);
			$html.= sprintf('<td align="left" valign="top"><code><a href="#%s">%s</a></code></td>%s', 
												$variable,
												$variable,
												$this->CR_HTML
											);
			$html.= sprintf('<td align="left" valign="top">%s</tr>%s',												
												$shortdesc,
												$this->CR_HTML
											);
		}
		
		return $html;
	} // end func getVariableSummaryParagraph
	
	/**
	* Returns a paragraph for the variable detail list. 
	* @param	string	$label
	* @param	array		$variables
	* @access	private
	* @see		getMethodlist(), getVariablelist()
	*/	
	function getVariableDetailParagraph($label, $variables) {
		
		$html = sprintf("<b>%s</b>%s", $label, $this->CR_HTML);
		
		reset($variables);
		while(list($k, $varname)=each($variables)) 
			$html.= $this->variableRenderer->getVariableDetail($varname, $this->classdata["variables"][$varname]);
		
		return $html;	
	} // end func getVariableDetailParagraph
	
	/**
	* Returns the "path" of a class as a string.
	* @return	string	$path
	* @access	private
	*/
	function getClassPathString() {
	
		$path = "";
		
		krsort($this->tree["path"]);
		while (list($k, $parentclass)=each($this->tree["path"])) 
			$path.= $parentclass."/";
		$path = substr($path, 0, -1);

		return $path;
	} // end func getClassPathString
	
	/**
	* Returns a paragraph for the method summary list. 
	* @param	string	$label
	* @param	array		$functions
	* @access	private
	* @see		getFunctionlist()
	*/	
	function getFunctionSummaryParagraph($label, $functions) {
		
		$html = sprintf('<tr><td colspan="3" align="left" valign="top"><b>%s</b></td></tr>%s',
												$label,
												$this->CR_HTML
											);
											
		reset($functions);
		while (list($k, $function)=each($functions)) {
			list($return, $overrides, $shortdesc)= $this->functionRenderer->getFunctionSummary($function, $this->classdata["functions"][$function]);
			$html.= sprintf('<tr>%s<td align="left" valign="top">%s</td>%s',
												$this->CR_HTML,
												$return, 
												$this->CR_HTML
											);
			$html.= sprintf('<td align="left" valign="top"><code><a href="#%s()">%s()</a> %s</code></td>%s', 
												strtolower($function),
												$function,
												$overrides,
												$this->CR_HTML
											);
			$html.= sprintf('<td align="left" valign="top">%s</tr>%s',												
												$shortdesc,
												$this->CR_HTML
											);
		}
		
		return $html;
	} // end func getMethodSummaryParagraph
	
	/**
	* Renders a paragrah of methods in detail view
	* @param	string	$label
	* @param	array		$functions
	* @return	string	$html
	* @access	private
	*/
	function getFunctionDetailParagraph($label, $functions) {
	
		$html = sprintf("<b>%s</b>%s", $label, $this->CR_HTML);
		reset($functions);
		while(list($k, $function)=each($functions)) 
			$html.= $this->functionRenderer->getFunctionDetail($function, $this->classdata["functions"][$function]);
		
		return $html;	
	} // end func getFunctionDetailParagraph
	
	/**
	* Returns an array with the names of all public, private 
	* or undocumented functions in the current class.
	* @param	string	$type
	* @return	array		$sorted_functions
	* @access	private
	*/
	function getSortedFunctions($type) {
		if (!isset($this->classdata["sorted_functions"][$type]))
			return array();
		return $this->classdata["sorted_functions"][$type];
	} // end getSortedFunctions() 
	
	/**
	* Returns an array with the names of all public, private
	* or undocumented variables in the current class
	* @param	string	$type
	* @return	array		$sorted_variables
	* @access	private
	*/
	function getSortedVariables($type) {
		if (!isset($this->classdata["sorted_variables"][$type]))
			return array();
		return $this->classdata["sorted_variables"][$type];		
	} // end func getSortedVariables
		
} // end class phpdoc_renderer_classes
?>